;;
;; Copyright (c) 2005, Gigamonkeys Consulting All rights reserved.
;;

(in-package :com.gigamonkeys.markup)

(defvar *preprocessors* (list 'strip-invisible))

(defun render (output-type input-file &optional (output-file (output-filename output-type input-file)))
  (let ((sexp (parse-file input-file)))
    (render-sexp output-type sexp output-file)))

(defun output-filename (output-type input-file)
  (make-pathname :type (string-downcase output-type) :defaults input-file))

(defmacro with-tag-translations ((&rest translations) &body body)
  `(let ((*tag-translations* (append ',translations *tag-translations*)))
     ,@body))

(defun render-sexp (output-type sexp output-file)
  (loop for preprocessor in *preprocessors* do
       (setf sexp (funcall preprocessor sexp)))
  (render-as output-type sexp output-file)
  (truename output-file))

(defgeneric render-as (output-type sexp output-file))

(defun extract-notes (book)
  (let ((note-counter 0)
        (notes nil))
    (labels ((walk (sexp)
               (cond
                 ((atom sexp) sexp)
                 ((eql (first sexp) ':note)
                  (push sexp notes)
                  (list ':note-ref (incf note-counter)))
                 (t (mapcar #'walk sexp)))))
      (values (mapcar #'walk book) (nreverse notes)))))

(defun invisible-p (p)
  "Elements that are not output at all."
  (declare (optimize (safety 3)))
  (and (consp p)
       (or (member (first p) *invisible*)
	   (and (member (first p) *invisible-with-dot*)
		(and (stringp (second p))
		     (plusp (length (second p))) ; kludge--somehow I was
					; getting (:p "") which of
					; course made the next
					; expression barf.
		     (char= (char (second p) 0) #\.))))))

(defun strip-invisible (sexp)
  (if (atom sexp)
    sexp
    (mapcar #'strip-invisible (remove-if #'invisible-p sexp))))


